/*********************************************************************
*                SEGGER MICROCONTROLLER SYSTEME GmbH                 *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 1996 - 2004  SEGGER Microcontroller Systeme GmbH        *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

***** emWin - Graphical user interface for embedded applications *****
emWin is protected by international copyright laws.   Knowledge of the
source code may not be used to write a similar product.  This file may
only be used in accordance with a license and should not be re-
distributed in any way. We appreciate your understanding and fairness.
----------------------------------------------------------------------
File        : WM_TOUCH_Calibrate.c
Purpose     : Demonstrates how a touch screen can be calibrated at run time
Requirements: WindowManager - (X)
              MemoryDevices - ( )
              AntiAliasing  - ( )
              VNC-Server    - ( )
              PNG-Library   - ( )
              TrueTypeFonts - ( )
---------------------------END-OF-HEADER------------------------------
*/

#include "DIALOG.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define NUM_CALIB_POINTS  5  // Number of points for calibration

/*********************************************************************
*
*       Static Data
*
**********************************************************************
*/
static int _aRefX[NUM_CALIB_POINTS];
static int _aRefY[NUM_CALIB_POINTS];
static int _aSamX[NUM_CALIB_POINTS];
static int _aSamY[NUM_CALIB_POINTS];

/*********************************************************************
*
*       Static code
*
**********************************************************************
*/
/*********************************************************************
*
*       _cbBk
*/
static void _cbCalibrate(WM_MESSAGE * pMsg) {
  static int      xSize, ySize;
  GUI_PID_STATE * pState;
  static int      Pressed;
  static int      Index;

  switch(pMsg->MsgId) {
  case WM_CREATE:
    xSize = LCD_GetXSize();
    ySize = LCD_GetYSize();
    Index = 0;
    //
    // Calculate reference points depending on LCD size
    //
    _aRefX[0] = (xSize * 5) / 100;
    _aRefY[0] = (ySize * 5) / 100;
    _aRefX[1] = xSize - (xSize * 5) / 100;
    _aRefY[1] = _aRefY[0];
    _aRefX[2] = _aRefX[1];
    _aRefY[2] = ySize - (ySize * 5) / 100;
    _aRefX[3] = _aRefX[0];
    _aRefY[3] = _aRefY[2];
    _aRefX[4] = xSize / 2;
    _aRefY[4] = ySize / 2;
    break;
  case WM_PAINT:
    GUI_SetBkColor(GUI_WHITE);
    GUI_Clear();
    GUI_SetColor(GUI_BLACK);
    GUI_SetPenSize(3);
    GUI_DispStringHCenterAt("Please touch the point", LCD_GetXSize() / 2, LCD_GetYSize() / 2 - 60);
    GUI_DrawCircle(_aRefX[Index], _aRefY[Index], 5);
    break;
  case WM_TOUCH:
    pState = (GUI_PID_STATE *)pMsg->Data.p;
    if(pState->Pressed) {
      Pressed = 1;
    } else if (Pressed) {
      //
      // Store sample points
      //
      _aSamX[Index] = GUI_TOUCH_GetxPhys();
      _aSamY[Index] = GUI_TOUCH_GetyPhys();
      Index++;
      WM_InvalidateWindow(pMsg->hWin);
      if (Index == NUM_CALIB_POINTS) {
        //
        // Pass measured points to emWin
        //
        GUI_TOUCH_CalcCoefficients(NUM_CALIB_POINTS, _aRefX, _aRefY, _aSamX, _aSamY, xSize, ySize);
        WM_DeleteWindow(pMsg->hWin);
      }
    }
    break;
  }
}

/*********************************************************************
*
*       _cbBk
*/
static void _cbBk(WM_MESSAGE * pMsg) {
  int     Id, NCode;
  WM_HWIN hButton;

  switch(pMsg->MsgId) {
  case WM_PAINT:
    GUI_SetBkColor(GUI_BLACK);
    GUI_Clear();
    //
    // Create button to start calibration.
    //
    hButton = BUTTON_CreateEx(10, 10, 90, 30, pMsg->hWin, WM_CF_SHOW, 0, GUI_ID_BUTTON0);
    BUTTON_SetText(hButton, "Calibrate");
    break;
  case WM_NOTIFY_PARENT:
    Id    = WM_GetId(pMsg->hWinSrc);
    NCode = pMsg->Data.v;
    switch (Id) {
    case GUI_ID_BUTTON0:
      switch(NCode) {
      case WM_NOTIFICATION_RELEASED:
        //
        // Create touch calibration window.
        //
        WM_CreateWindowAsChild(0, 0, LCD_GetXSize(), LCD_GetYSize(), pMsg->hWin, WM_CF_SHOW, _cbCalibrate, 0);
        break;
      }
      break;
    }
    break;
  }
}

/*********************************************************************
*
*       Public code
*
**********************************************************************
*/
/*********************************************************************
*
*       MainTask
*/
void MainTask(void) {
  GUI_Init();
  GUI_CURSOR_Show();

  WM_CreateWindow(0, 0, LCD_GetXSize(), LCD_GetYSize(), WM_CF_SHOW, _cbBk, 0);

  while (1) {
    GUI_Delay(100);
  }
}

/*************************** End of file ****************************/